::cisco::eem::event_register_timer watchdog time 10 name "easy_ipsla_tracker_timer"
#-
# Copyright (c) 2009 Joe Marcus Clarke <jclarke@cisco.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of Cisco, the name of the copyright holder nor the
#    names of their respective contributors may be used to endorse or
#    promote products derived from this software without specific prior
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#
# TECHNICAL ASSISTANCE CENTER (TAC) SUPPORT IS NOT AVAILABLE FOR THIS SCRIPT.
# For questions or help, send email to ask-easy@cisco.com.
#
# This policy tracks an IP SLA operation to see if it goes down.  When
# it does, it will execute a list of commands, and optionally send a
# syslog message.
#
# This policy uses the following environment variables:
#
# easy_ipsla_track_idx     : Tracked object ID
#
# easy_ipsla_down_cmds     : Semi-colon separated list of commands to run when
#                            the tracked object goes down
#
# easy_ipsla_up_cmds       : Semi-colon separated list of commands to run when
#                            the tracked object comes back up
#
# easy_ipsla_down_syslog   : Syslog message to send when the tracked object
#                            goes down (or empty for no message)
#
# easy_ipsla_up_syslog     : Syslog message to send when the tracked object
#                            comes back up (or empty for no message)
#
# easy_ipsla_down_slg_prio : Priority of the syslog message sent when the
#                            tracked object goes down
#
# easy_ipsla_up_slg_prio   : Priority of the syslog message sent when the
#                            tracked object comes back up
#
# $Id: tm_easy_ipsla_tracker.tcl,v 1.4 2009/05/19 19:50:32 marcus Exp $
#

if { ![info exists easy_ipsla_track_idx] } {
    set result "ERROR: Policy cannot be run: variable easy_ipsla_track_idx has not been set"
    error $result $errorInfo
}

if { ![info exists easy_ipsla_down_cmds] } {
    set result "ERROR: Policy cannot be run: variable easy_ipsla_down_cmds has not been set"
    error $result $errorInfo
}

if { ![info exists easy_ipsla_up_cmds] } {
    set result "ERROR: Policy cannot be run: variable easy_ipsla_up_cmds has not been set"
    error $result $errorInfo
}

if { ![info exists easy_ipsla_down_syslog] } {
    set result "ERROR: Policy cannot be run: variable easy_ipsla_down_syslog has not been set"
    error $result $errorInfo
}

if { ![info exists easy_ipsla_up_syslog] } {
    set result "ERROR: Policy cannot be run: variable easy_ipsla_up_syslog has not been set"
    error $result $errorInfo
}

if { ![info exists easy_ipsla_down_slg_prio] } {
    set result "ERROR: Policy cannot be run: variable easy_ipsla_down_slg_prio has not been set"
    error $result $errorInfo
}

if { ![info exists easy_ipsla_up_slg_prio] } {
    set result "ERROR: Policy cannot be run: variable easy_ipsla_up_slg_prio has not been set"
    error $result $errorInfo
}

namespace import ::cisco::eem::*
namespace import ::cisco::lib::*

proc run_cli { clist } {
    set rbuf ""

    if {[llength $clist] < 1} {
	return -code ok $rbuf
    }

    if {[catch {cli_open} result]} {
        return -code error $result
    } else {
	array set cliarr $result
    }

    if {[catch {cli_exec $cliarr(fd) "enable"} result]} {
        return -code error $result
    }

    foreach cmd $clist {
	if {[catch {cli_exec $cliarr(fd) $cmd} result]} {
            return -code error $result
	}

	append rbuf $result
    }

    if {[catch {cli_close $cliarr(fd) $cliarr(tty_id)} result]} {
        puts "WARNING: $result"
    }

    return -code ok $rbuf
}

set prio_array(emergencies) "emerg"
set prio_array(alerts) "alert"
set prio_array(critical) "crit"
set prio_array(errors) "err"
set prio_array(warnings) "warning"
set prio_array(notifications) "notice"
set prio_array(informational) "info"
set prio_array(debugging) "debug"

if { [catch { set prev_state [context_retrieve EASYIPSLACTXT prev_state] } errmsg] } {
    set prev_state "Up"
}

set output [run_cli [list "show track $easy_ipsla_track_idx | incl Reachability is"]]
if { $output == "" } {
    action_syslog priority err msg "ERROR: Did not get output from 'show track $easy_ipsla_track_idx | incl Reachability is'"
    return -code ok
}

if { ! [regexp {Reachability is (\w+)} $output -> status] } {
    action_syslog priority err msg "ERROR: Output from 'show track $easy_ipsla_track_idx | incl Reachability is' is not in expected format (output = '$output')"
    return -code ok
}
if { $status == $prev_state } {
    set prev_state $status
    catch { context_save EASYIPSLACTXT prev_state } errmsg
    return -code ok
}

set cmds $easy_ipsla_down_cmds
set slg $easy_ipsla_down_syslog
set prio $prio_array($easy_ipsla_down_slg_prio)

if { $status == "Up" } {
    set cmds $easy_ipsla_up_cmds
    set slg $easy_ipsla_up_syslog
    set prio $prio_array($easy_ipsla_down_slg_prio)
}

set cout ""
if { [catch {run_cli [split $cmds ";"]} result] } {
    set cout "Failed to run commands: '$result'"
} else {
    set cout $result
}

if { $slg != "" } {
    append slg " (cmd output: $cout)"
    action_syslog priority $prio msg $slg
}

set prev_state $status
catch { context_save EASYIPSLACTXT prev_state } errmsg
